'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _ajv = require('ajv');

var _ajv2 = _interopRequireDefault(_ajv);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createValidators(definitions) {
    var ajv = new _ajv2.default({ allErrors: true });
    var validators = {};

    var _loop = function _loop(errorKey) {
        var definition = definitions[errorKey];
        var errorName = errorKey;

        var validateSchema = ajv.compile(definition);

        validators[errorKey] = function (data) {
            var isValid = validateSchema(data);

            if (!isValid) {
                var messages = '  - ';

                messages += ajv.errorsText(validateSchema.errors, {
                    separator: '\n  - '
                });

                throw new Error(errorName + ' \'data\' are invalid:\n' + messages);
            }
        };
    };

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = Object.keys(definitions)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var errorKey = _step.value;

            _loop(errorKey);
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return validators;
}

var DEFAULT_OPTIONS = {
    validate: true
};

function createDefinitions(definitions) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : DEFAULT_OPTIONS;

    var validate = options.validate;
    var errorDefinitions = {};
    var validators = createValidators(definitions);

    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = Object.keys(definitions)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var errorKey = _step2.value;

            var definition = definitions[errorKey];

            errorDefinitions[errorKey] = _extends({}, definition, {
                name: definition.name || errorKey
            });

            if (validate && validators[errorKey]) {
                errorDefinitions[errorKey].validate = validators[errorKey];
            }

            if (!definition.type) {
                definition.type = 'object';
            }
        }
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    return errorDefinitions;
}

exports.default = createDefinitions;